/**
 * Copyright (c) 2014-2016 Zuoyebang, All rights reseved.
 * @fileoverview app端绘图工具
 * @author wangyan01 | wangyan01@zuoyebang.com
 * @version 1.0 | 2016-09-07 | wangyan01    // 初始版本。
 * @description
 *    interface
 *      headerMarginTop {Number}    // 参数（公共）：图表margin-top值（默认0）
 *      headerMarginBottom {Number} // 参数（公共）：图表margin-bottom值（默认0）
 *      opacity {Number}            // 参数（公共）：图表透明度（默认1，1：不透明，0：完全透明）
 *      title {Object}
 *        fontSize {Number}         // 参数（公共）：图表title 字体大小（默认12px）
 *        text {String}             // 参数（公共）：图表title 内容
 *        color {String}            // 参数（公共）：图表title 颜色
 *        textAlign {String}        // 参数（公共）：图表title的文字浮动（默认'center'，还有 'left'/'right）
 *      =================
 *      type: "polyline"            // 参数：绘制类型——折线
 *      categories {Array[String]}  // 参数：折线图 x 轴标签集合(必选)
 *      dataSet {Array[String]}     // 参数：折线数据组(必选，为二维数组，每个一维数组为一条折线数据)
 *      widthPercent {Number}       // 参数：图表绘制区宽(百分比 eg: 0.2)
 *      resolution {Number}         // 参数：图表绘制区高/宽(eg: 16/9)
 *      isNoAxisX {Boolean}         // 参数：图表是否隐藏x轴线(默认显示)
 *      isNoAxisY {Boolean}         // 参数：图表是否隐y轴线(默认显示)
 *      line {Object}               // 参数：折线配置选项(可选)
 *        color {String}            // 参数：折线颜色(eg: #CCC)
 *        stroke {Object}           // 参数：折线线条配置选项(可选)
 *          width {Number}          // 参数：折线线条宽度(可选，默认值：2)
 *        dot {Object|Boolean}      // 参数：折线点配置选项(可选，参阅下文详述)
 *          radius {Number}         // 参数：折线点半径(可选，默认值：线条宽度的 3 倍)
 *          color {String}          // 参数：折线点颜色
 *          hightLight {Array}      // 参数：第n个折线点实色[]
 *      xAxis {Object}              // 参数：x 轴选项(可选)
 *        isSHow {Boolean}          // 参数：x 轴是否展现(可选，默认值：true)
 *        orientation {String}      // 参数：x 轴方向(可选，取值范围：TOP|BOTTOM，默认值：BOTTOM)
 *        text {Object}             // 参数：x 轴文字配置选项(可选)
 *          fontSize {Number}       // 参数：x 轴文字尺寸(可选，默认值：14)
 *          color {String}          // 参数：x 轴文字色值(可选，默认值：'#666')
 *        stroke {Object}           // 参数：x 轴线条配置选项(可选)
 *          width {Number}          // 参数：x 轴线条宽度(可选，默认值：1)
 *          color {string}          // 参数：x 轴线条色值(可选，默认值：'#999')
 *      yAxis {Object}              // 参数：y 轴配置选项(可选)
 *        isShow {Boolean}          // 参数：y 轴是否展现(可选，默认值：false)
 *        orientation {String}      // 参数：y 轴方向(可选，取值范围：LEFT|RIGHT，默认值：LEFT)
 *        division {Number}         // 参数：y 轴显示刻度数(可选，默认值：0)
 *        stroke {Object}           // 参数：y 轴线条配置选项(可选)
 *          width {Number}          // 参数：y 轴线条宽度(可选，默认值：1)
 *          color {string}          // 参数：y 轴线条色值(可选，默认值：'#999')
 *        text {Object}             // 参数：y 轴文字配置选项(可选)
 *          fontSize {Number}       // 参数：y 轴文字尺寸(可选，默认值：14)
 *          color {String}          // 参数：y 轴文字色值(可选，默认值：'#666')
 *        unit {Number}             // 参数：y 轴显示刻度单位(可选，默认值：1)
 *      ==================
 *      type: "pie"                 // 参数：绘制类型——环形图
 *      pieRPercent {Number}        // 参数：饼图半径占屏幕宽百分比，即半径/bodywidth(必选，屏幕款的百分比 eg: 0.4)。
 *      donutWidth {Number}         // 参数：环图的内环与外环间的宽度(可选，默认值：0；为0时为饼状图)。
 *      slices {Array}              // 参数：饼图切片数组(可选，默认仅包含一个 360° 的切片)。
 *        color {String}            // 参数：切片颜色(可选，默认从 16 色预定义色谱中按序输出)。
 *        percent {Number}          // 参数：切片百分比(用于计算切片角度，可选，默认为0).
 *        desc {String}             // 参数：切片描述（可选，默认为空）
 *      descLineHeight {Number}     // 参数：切片描述行间距（可选，默认1.5）
 */

var $ = require('common:widget/lib/zepto/zepto.js'),
  zybLog = require('common:widget/util/zybLog.js'),
  Stage = require('lts-activity:widget/canvas4app/stage.js'),
  Polyline = require('common:widget/ui/polyline/polyline.js');

var body = document.body.getBoundingClientRect(),
  scale = 1,
  resolution = 9/16;

var bodyWidth = body.width * scale;

var pieColor = ['#EA7F77', '#F2A473', '#F8E765', '#8AD777', '#EEE'];

function paint(data) {
  var stage = new Stage(
    data.type || 'polyline',
    $('.stage')
  );
  data = $.extend(true, {
    title: {
      fontSize: '18'
    }
  }, data);
  var header = document.createElement('header');
  header.innerText = data.title.text || '';
  var headerMarginTop = data.headerMarginTop || 0,
    headerMarginBottom = data.headerMarginBottom || 0;
  // 设置图表头信息
  $(header).css({
    'fontSize': data.title.fontSize + 'px',
    'marginTop': headerMarginTop + 'px',
    'marginBottom': headerMarginBottom + 'px',
    'textAlign': data.title.textAlign || 'center',
    'color': data.title.color || '#EEE'
  });
  // 设置图表透明度
  $('.stage').css({
    'opacity': data.opacity || 1,
    '-webkit-opacity': data.opacity || 1
  });

  data.title.text && $('.stage').append(header);
  stage.paint(getPaintData(data));
}

function getPaintData(data) {
  var paintData = null;
  switch(data.type) {
    case 'polyline':
      var widthPercent = data.widthPercent || 1,
        resolution = data.resolution || resolution;
      var oriY = data.yAxis.orientation || 'left';
      var canvasSize = {
        width: bodyWidth * widthPercent * (oriY.toLowerCase() == 'left' ? .95 : 1),    // 这里*.95是为了生成polygon时，最后一个点不会被overflow掉，polyline创建完成后，需要恢复svgroot的原始尺寸
        height: bodyWidth * widthPercent * resolution
      };
      paintData = $.extend(true, {}, {
        chartSize: {
          width: canvasSize.width - 30,
          height: canvasSize.height - 30
        },
        canvasSize: canvasSize
      }, data);
      break;
    case 'pie':
      var pieR = data.pieRPercent ? bodyWidth * data.pieRPercent : bodyWidth,
        donutR = data.donutWidth ? pieR - data.donutWidth : 0;
      var slices = new Array();
      paintData = $.extend(true, {}, {
        pieR: pieR,           // 外径
        donutR: donutR        // 内径与外径间距
      }, pieSlicesProcess(data));
      break;
    default: 
      break;
  }
  return paintData;
}

function pieSlicesProcess(data) {
  if(data.slices) {
    for(var i = 0; i < Math.min(pieColor.length, data.slices.length); i++) {
      data.slices[i].color = data.slices[i].color || pieColor[i];
    }
  }
  return data;
}