define('lts-activity:widget/canvas4app/piePlugin.js', function(require, exports, module){ /**
 * Copyright (c) 2014-2016 Zuoyebang, All rights reseved.
 * @fileoverview app端绘图工具 pie扩展
 * @author wangyan01 | wangyan01@zuoyebang.com
 * @version 1.0 | 2016-09-07 | wangyan01    // 初始版本。
 * @description
 *    本扩展在原有pie基础上进行样式补充
 */

var Pie = require('common:widget/ui/pie/pie.js');

// 'A 5 5 -90 0 1 x y';
function PiePlugin(pie, descs) {
  if(!pie instanceof Pie) {
    throw TypeError('must be a Pie');
  }
  this.source = pie;
  this.node = null;
}

PiePlugin.prototype.getNode = function() {
  return this.node;
}

PiePlugin.prototype.addDesc = function() {
  var parent = this.node || this.source.getNode();
  var extDom = createDesc(getSliceColors(parent), this.source.args.slices);
  var pieWrapper = document.createElement('div');
  parent.appendTo(pieWrapper);
  extDom.appendTo(pieWrapper);

  this.node = $(pieWrapper).addClass('ext-pie');
  return this;
}

PiePlugin.prototype.addArc = function() {
  var parent = this.node || this.source.getNode();
  var arcR = (this.source.args.outerR - this.source.args.innerR)/2;
  $('[clip-path] path', parent).each(function(index, ele) {
    var d = $(ele).attr('d');
    $(ele).attr('d', d.replace(/L ([-\w.]*? [-\w.]*?)/, 'A ' + arcR + ' ' + arcR + ' 90 0 1 $1').replace(/L ([-\w.]*? [-\w.]*?)/, 'A ' + arcR + ' ' + arcR + ' 90 0 0 $1'));
  });

  this.node = parent;
  return this;
}

PiePlugin.prototype.descLineHeight = function(lineHeight) {
  var parent = this.node || this.source.getNode();
  $('li', parent).css('lineHeight', lineHeight);

  this.node = parent;
  return this;
}

/*
 * @param data: [{color, percent}]
 */
function createDesc(colors, slices) {
  var ul = document.createElement('ul');
  var MIN = 0.1;
  for(var i = 0; i < colors.length; i++) {
    var li = document.createElement('li');
    var em = document.createElement('em');
    var em1 = em.cloneNode(),
      em2 = em.cloneNode();
    $(em1).addClass('bg-color').css('background', colors[i]).appendTo(li);
    var percents = (slices[i].percent * 100).toFixed(1);
    percents < MIN && (percents = '< ' + MIN);
    $(em2).addClass('percents').text(percents + '% ' + (slices[i].desc || '')).appendTo(li);
    $(li).appendTo(ul);
  }
  return $(ul);
}

function getSliceColors(node) {
  var colors = [];
  $('[slice-index]', node).each(function(index, ele) {
    colors.push($(ele).css('fill'));
  });
  return colors;
}

module.exports = PiePlugin; 
});
;define('lts-activity:widget/canvas4app/polylinePlugin.js', function(require, exports, module){ /**
 * Copyright (c) 2014-2016 Zuoyebang, All rights reseved.
 * @fileoverview app端绘图工具 polyline扩展
 * @author wangyan01 | wangyan01@zuoyebang.com
 * @version 1.0 | 2016-09-07 | wangyan01    // 初始版本。
 * @description
 *    本扩展在原有polyline基础上进行样式补充
 */

var Polyline = require('common:widget/ui/polyline/polyline.js');

function PolylinePlugin(polyline) {
  if (!polyline instanceof Polyline) {
    throw TypeError('must be a Polyline');
  }
  this.source = polyline;
  this.node = polyline.getNode();

  this.xAxis = {
    x1: $('.wgt-polyline-axis-x', this.node).attr('x1'),
    y1: $('.wgt-polyline-axis-x', this.node).attr('y1'),
    x2: $('.wgt-polyline-axis-x', this.node).attr('x2'),
    y2: $('.wgt-polyline-axis-x', this.node).attr('y2')
  },
  this.yAxis = {
    x1: $('.wgt-polyline-axis-y', this.node).attr('x1'),
    y1: $('.wgt-polyline-axis-y', this.node).attr('y1'),
    x2: $('.wgt-polyline-axis-y', this.node).attr('x2'),
    y2: $('.wgt-polyline-axis-y', this.node).attr('y2')
  };
}

PolylinePlugin.prototype.getNode = function() {
  return this.node;
}

PolylinePlugin.prototype.startFromZero = function() {
  var self = this;
  var parent = self.node || self.source.getNode();
  var orientation = self.source.yAxis.orientation == 'LEFT' ? 1 : -1;

  // calculate
  var unit = (self.xAxis.x2 - self.xAxis.x1) / (self.source.categories.length - 1);
  // xAxis text
  var xAxisTexts = $('.wgt-axis-x-group text', parent);
  xAxisTexts.each(function(index, ele) {
    if(index == 0) {
      $(ele).attr('x', self.xAxis.x1);
    } else {
      $(ele).attr('x', parseFloat(self.xAxis.x1) + unit * index);
    }
    if(index == xAxisTexts.length - 1 && orientation == -1) {
      $(ele).attr('text-anchor', 'end');
    }
  });
  // xAxis divisions
  var xAxisDivsions = $('.wgt-axis-x-group line:not(.wgt-polyline-axis-x)', parent);
  xAxisDivsions.each(function(index, ele) {
    if(index == 0) {
      $(ele).attr('x1', self.xAxis.x1);
      $(ele).attr('x2', self.xAxis.x1);
    } else {
      $(ele).attr('x1', parseFloat(self.xAxis.x1) + unit * index);
      $(ele).attr('x2', parseFloat(self.xAxis.x1) + unit * index);
    }
  });
  // wgt-polyline-group
  $('.wgt-polyline-group g >*', parent).each(function(index, ele) {
    switch (ele.nodeName.toLowerCase()) {
      case 'polygon':
        var points = $(ele).attr('points').split(' ');
        // 1，2组在轴线上，到最后两组点结束（最后两组x轴一致，即多边形最后一条边）
        for (var i = 0; i < points.length; i++) {
          var times = i <= 1 ? 0 : (i == points.length - 1 ? i - 2 : i - 1);
          points[i] = points[i].replace(/([-\w.]*?),/g, function(match, m1) {
            return (parseFloat(self.xAxis.x1) + unit * times) + ',';
          });
        }
        $(ele).attr('points', points.join(' '));
        break;
      case 'polyline':
        var points = $(ele).attr('points').split(' ');
        for (var j = 0; j < points.length; j++) {
          var times = j;
          points[j] = points[j].replace(/([-\w.]*?),/g, function(match, m1) {
            return (parseFloat(self.xAxis.x1) + unit * times) + ',';
          });
        }
        $(ele).attr('points', points.join(' '));
        break;
      case 'circle':
        var cx = $(ele).attr('cx');
        $(ele).attr('cx', parseFloat(self.xAxis.x1) + unit * ($(ele).index() - 2));
        break;
    }
  });

  // 调整y轴text
  if(orientation == -1) {
    self.yAxisTextInside();
  } else {
    self.resetRootSize();
  }


  this.node = parent;
  return this;
}

PolylinePlugin.prototype.addMask = function() {
  var parent = this.node || this.source.getNode();
  $('polygon', parent).attr('mask', 'url(#Mask)');

  this.node = parent;
  return this;
}

PolylinePlugin.prototype.noAxisYDivisions = function() {
  var parent = this.node || this.source.getNode();
  $('.wgt-axis-y-group line:not(.wgt-polyline-axis-y)', parent).each(function(index, ele) {
    svgEleAddClass(ele, 'hidden');
  });

  this.node = parent;
  return this;
}

PolylinePlugin.prototype.noAxisXDivisions = function() {
  var parent = this.node || this.source.getNode();
  $('.wgt-axis-x-group line:not(.wgt-polyline-axis-x)', parent).each(function(index, ele) {
    svgEleAddClass(ele, 'hidden');
  });

  this.node = parent;
  return this;
}

PolylinePlugin.prototype.noAxisXLine = function() {
  var parent = this.node || this.source.getNode();
  $('.wgt-axis-x-group .wgt-polyline-axis-x', parent).each(function(index, ele) {
    svgEleAddClass(ele, 'hidden');
  });

  this.node = parent;
  return this;
}

PolylinePlugin.prototype.noAxisYLine = function() {
  var parent = this.node || this.source.getNode();
  $('.wgt-axis-y-group .wgt-polyline-axis-y', parent).each(function(index, ele) {
    svgEleAddClass(ele, 'hidden');
  });

  this.node = parent;
  return this;
}

PolylinePlugin.prototype.setHighlight = function(array) {
  var parent = this.node || this.source.getNode();
  $('[dot-index]', parent).each(function(index, ele) {
    if(~array.indexOf(index)) {
      var self = $(ele);
      var strokeColor = self.attr('stroke'),
        fillColor = self.attr('fill');
      self.css({
        fill: strokeColor,
        stroke: fillColor,
        r: parseFloat(self.attr('r')) + 2
      });
    }
  });

  this.node = parent;
  return this;
}

PolylinePlugin.prototype.yAxisTextInside = function() {
  var parent = this.node || this.source.getNode();
  var oriY = this.source.yAxis.orientation.toLowerCase() == 'left' ? 1 : -1;
  // 调整y轴文字位置，y轴内侧距y轴10px处
  var margin = 10;
  var yAxisLineX1 = $('.wgt-polyline-axis-y', parent).attr('x1');
  $('.wgt-axis-y-group text', parent).each(function(index, ele) {
    $(ele).attr({
      'x': parseFloat(yAxisLineX1) + margin * oriY,
      'text-anchor': 'end'
    });
  });
  var xAxisWidth = this.xAxis.x2 - this.xAxis.x1;

  // canvas重新居中
  $('.wgt-polyline-canvas', parent).attr('transform', 'translate('
    + (this.source.canvasSize.width / 2 + margin * 2)
    + ','
    + (this.source.canvasSize.height / 2)
    + ')');

  this.node = parent;
  return this;
}

// 重计算svg root的尺寸，考虑边界点被overflow掉，创建polyline时size*.95生成canvas，现在恢复svg原始尺寸，hack
PolylinePlugin.prototype.resetRootSize = function() {
  var parent = this.node || this.source.getNode();
  var rootWidth = parent.attr('width');
  parent.attr('width', rootWidth / .95);

  this.node = parent;
  return this;
}

PolylinePlugin.prototype.setPolylineColor = function(color) {
  var parent = this.node || this.source.getNode();
  $('polyline', parent).attr('stroke', color);

  this.node = parent;
  return this;
}

PolylinePlugin.prototype.setDotColor = function(color) {
  var parent = this.node || this.source.getNode();
  $('circle', parent).each(function(index, ele) {
    $(ele).attr('stroke', color);
  });

  this.node = parent;
  return this;
}

// 对原始数据做处理，使得生成的y轴刻度为正数
// 这里使用: n ≥ max(dataSet) && (n - min(dataSet))% division == 0
PolylinePlugin.prototype.unitToFixed0 = function(setting) {
  var dataSet = setting.dataSet,
    division = setting.yAxis.division;
  for(var i = 0; i < dataSet.length; i++) {
    var max = Math.max.apply({}, dataSet[i]),
      min = Math.min.apply({}, dataSet[i]);
    if((max - min) % division != 0) {
      setting.dataSet[i].push(max + (division - (max - min) % division));
    }
  }
}

function svgEleAddClass(ele, classNameArray) {
  if(!(classNameArray instanceof Array)) {
    classNameArray = [classNameArray];
  }
  var className = ele.getAttribute('class');
  var addList = [];
  for(var i = 0; i < classNameArray.length; i++) {
    if(!new RegExp(classNameArray[i]).test(className)) {
      addList.push(classNameArray[i]);
    }
  }
  ele.setAttribute('class', className + ' ' + addList.join(' '));
}

module.exports = PolylinePlugin; 
});
;define('lts-activity:widget/canvas4app/stage.js', function(require, exports, module){ /**
 * Copyright (c) 2014-2016 Zuoyebang, All rights reseved.
 * @fileoverview app端绘图工具
 * @author wangyan01 | wangyan01@zuoyebang.com
 * @version 1.0 | 2016-09-07 | wangyan01    // 初始版本。
 */

var Polyline = require('common:widget/ui/polyline/polyline.js'),
  PolylinePlugin = require('lts-activity:widget/canvas4app/polylinePlugin.js'),
  Pie = require('common:widget/ui/pie/pie.js'),
  PiePlugin = require('lts-activity:widget/canvas4app/piePlugin.js');

var painters = {
  'polyline': Polyline,
  'pie': Pie
};

var _setting_polyline = {
  line: {
    isShowShadow: true,
    dot: {
      // radius: 5,
      isHollow: true
    }
  },
  xAxis: {
    stroke: {
      color: '#45B7FF',
      width: .5
    }
  },
  yAxis: {
    isShow: true,
    division: 4,
    stroke: {
      color: '#45B7FF',
      width: .5
    },
    text: {
      formatter: function(e) {
        return parseInt(e);
      }
    }
  },
  colorSet: ['#45B7FF']
},
_settint_pie = {
  rotation: -90,         // 旋转到从y轴正方向起始
  strokeColor: '#FFF',   // 使用白色描边
  animation: true        // 启用默认展现动画
};

function Stage(type, target) {
  if(!/polyline|pie/.test(type)) {
    throw TypeError('type is not supported');
  }
  if(!target instanceof HTMLElement || !target instanceof $) {
    throw TypeError('target is not a HTMLElement or a zepto object');
  }
  this.config = {
    type: type,
    target: target
  };
}

Stage.prototype.paint = function(data) {
  var setting = null;
  var target = this.config.target;
  var painter = null;
  data.yAxis = data.yAxis || {};
  var $html = null;
  switch(this.config.type) {
    case 'polyline':
      setting = $.extend(true, {}, _setting_polyline, {
        chartSize: data.chartSize,
        canvasSize: data.canvasSize,
        xAxis: data.xAxis,
        yAxis: $.extend(true, {}, data.yAxis, {
          text: {
            formatter: function(e) {
              return parseFloat(e * (data.yAxis.unit || 1)).toFixed(0);
            }
          }
        }),
        categories: data.categories,
        dataSet: data.dataSet
      }, data);
      PolylinePlugin.prototype.unitToFixed0(setting);
      painter = new painters[this.config.type](setting);
      var plugin = new PolylinePlugin(painter);
      data.isNoAxisX && plugin.noAxisXLine();
      data.isNoAxisY && plugin.noAxisYLine();
      data.line && data.line.color && plugin.setPolylineColor(data.line.color);
      data.line && data.line.dot && data.line.dot.color && plugin.setDotColor(data.line.dot.color);
      data.line && data.line.dot && data.line.dot.highLight && plugin.setHighlight(data.line.dot.highLight);
      $html = plugin
        .noAxisXDivisions()
        .noAxisYDivisions()
        .startFromZero()
        .addMask()
        .getNode();
      break;
    case 'pie':
      setting = data;
      painter = new painters[this.config.type](setting);
      var plugin = new PiePlugin(painter);
      $html = plugin.addArc().addDesc().descLineHeight(data.descLineHeight || 1.5).getNode();
      break;
    default:
      break;
  }
  target.append($html);
}

module.exports = Stage; 
});